@@ -1,14 +1,16 @@
 package Inline;
 
 use strict;
-require 5.005;
-$Inline::VERSION = '0.43';
+require 5.006;
+$Inline::VERSION = '0.47';
 
 use AutoLoader 'AUTOLOAD';
 use Inline::denter;
 use Config;
 use Carp;
 use Cwd qw(abs_path cwd);
+use File::Spec;
+use File::Spec::Unix;
 
 my %CONFIG = ();
 my @DATA_OBJS = ();
@@ -19,29 +21,39 @@ my $untaint = 0;
 my $safemode = 0;
 $Inline::languages = undef; #needs to be global for AutoLoaded error messages
 
-my %shortcuts = 
+our $did = '_Inline'; # Default Inline Directory
+
+# This is the config file written by create_config_file().
+our $configuration_file = 'config-' . $Config::Config{'archname'} . '-' . $];
+
+my %shortcuts =
   (
+   NOCLEAN =>      [CLEAN_AFTER_BUILD => 0],
    CLEAN =>        [CLEAN_BUILD_AREA => 1],
    FORCE =>        [FORCE_BUILD => 1],
    INFO =>         [PRINT_INFO => 1],
    VERSION =>      [PRINT_VERSION => 1],
-   NOCLEAN =>      [CLEAN_AFTER_BUILD => 0],
    REPORTBUG =>    [REPORTBUG => 1],
    UNTAINT =>      [UNTAINT => 1],
    SAFE =>         [SAFEMODE => 1],
    UNSAFE =>       [SAFEMODE => 0],
    GLOBAL =>       [GLOBAL_LOAD => 1],
+   NOISY =>        [BUILD_NOISY => 1],
+   TIMERS =>       [BUILD_TIMERS => 1],
+   NOWARN =>       [WARNINGS => 0],
    _INSTALL_ =>    [_INSTALL_ => 1],
    SITE_INSTALL => undef,  # No longer supported.
   );
 
-my $default_config = 
+my $default_config =
   {
    NAME => '',
    AUTONAME => -1,
    VERSION => '',
    DIRECTORY => '',
    WITH => [],
+   USING => [],
+
    CLEAN_AFTER_BUILD => 1,
    CLEAN_BUILD_AREA => 0,
    FORCE_BUILD => 0,
@@ -49,9 +61,14 @@ my $default_config =
    PRINT_VERSION => 0,
    REPORTBUG => 0,
    UNTAINT => 0,
+   NO_UNTAINT_WARN => 0,
    SAFEMODE => -1,
    GLOBAL_LOAD => 0,
+   BUILD_NOISY => 0,
+   BUILD_TIMERS => 0,
+   WARNINGS => 1,
    _INSTALL_ => 0,
+   _TESTING => 0,
   };
 
 sub UNTAINT {$untaint}
@@ -65,7 +82,8 @@ sub import {
 
     my $o;
     my ($pkg, $script) = caller;
-    $pkg =~ s/^.*[\/\\]//;
+    # Not sure what this is for. Let's see what breaks.
+    # $pkg =~ s/^.*[\/\\]//;
     my $class = shift;
     if ($class ne 'Inline') {
 	croak M01_usage_use($class) if $class =~ /^Inline::/;
@@ -107,7 +125,7 @@ sub import {
 	$o->{API}{language_id} = $language_id;
 	if ($option =~ /^(FILE|BELOW)$/ or
 	    not $option and
-	    defined $INC{"Inline/Files.pm"} and
+            defined $INC{File::Spec::Unix->catfile('Inline','Files.pm')} and
 	    Inline::Files::get_filename($pkg)
 	   ) {
 	    $o->read_inline_file;
@@ -152,7 +170,7 @@ sub bind {
       unless ($language_id =~ /^\S+$/ and $language_id !~ /\n/);
     $code = shift or croak M03_usage_bind();
     @config = @_;
-	
+
     my $next = 0;
     for (@config) {
 	next if $next++ % 2;
@@ -172,7 +190,7 @@ sub bind {
 #==============================================================================
 # Process delayed objects that don't have source code yet.
 #==============================================================================
-# This code is an ugly hack because of the fact that you can't use an 
+# This code is an ugly hack because of the fact that you can't use an
 # INIT block at "run-time proper". So we kill the warning for 5.6+ users
 # and tell them to use a Inline->init() call if they run into problems. (rare)
 my $lexwarn = ($] >= 5.006) ? 'no warnings;' : '';
@@ -243,7 +261,8 @@ sub glue {
        ) {
 	eval "require $o->{INLINE}{ILSM_module}";
 	croak M05_error_eval('glue', $@) if $@;
-	bless $o, $o->{INLINE}{ILSM_module};    
+        $o->push_overrides;
+	bless $o, $o->{INLINE}{ILSM_module};
 	$o->validate(@config);
     }
     else {
@@ -257,14 +276,63 @@ sub glue {
     }
     if ($o->{INLINE}{ILSM_suffix} ne 'so' and
 	$o->{INLINE}{ILSM_suffix} ne 'dll' and
+	$o->{INLINE}{ILSM_suffix} ne 'bundle' and
 	ref($o) eq 'Inline'
        ) {
 	eval "require $o->{INLINE}{ILSM_module}";
 	croak M05_error_eval('glue', $@) if $@;
+        $o->push_overrides;
 	bless $o, $o->{INLINE}{ILSM_module};
+	$o->validate(@config);
     }
-
     $o->load;
+    $o->pop_overrides;
+}
+
+#==============================================================================
+# Set up the USING overrides
+#==============================================================================
+sub push_overrides {
+    my ($o) = @_;
+    my ($language_id) = $o->{API}{language_id};
+    my ($ilsm) = $o->{INLINE}{ILSM_module};
+    for (@{$o->{CONFIG}{USING}}) {
+        my $using_module = /^::/
+                           ? "Inline::$language_id$_"
+                           : /::/
+                             ? $_
+                             : "Inline::${language_id}::$_";
+        eval "require $using_module";
+        croak "Invalid module '$using_module' in USING list:\n$@" if $@;
+        my $register;
+        eval "\$register = $using_module->register";
+        croak "Invalid module '$using_module' in USING list:\n$@" if $@;
+        for my $override (@{$register->{overrides}}) {
+            no strict 'refs';
+            next if defined $o->{OVERRIDDEN}{$ilsm . "::$override"};
+            $o->{OVERRIDDEN}{$ilsm . "::$override"} =
+              \&{$ilsm . "::$override"};
+            {
+            no warnings 'redefine';
+            *{$ilsm . "::$override"} =
+              \&{$using_module . "::$override"};
+            }
+        }
+    }
+}
+
+#==============================================================================
+# Restore the modules original methods
+#==============================================================================
+sub pop_overrides {
+    my $nowarn = $] >= 5.006 ? "no warnings 'redefine';" : '';
+    eval ($nowarn .
+    'my ($o) = @_;
+    for my $override (keys %{$o->{OVERRIDDEN}}) {
+        no strict "refs";
+        *{$override} = $o->{OVERRIDDEN}{$override};
+    }
+    delete $o->{OVERRIDDEN};')
 }
 
 #==============================================================================
@@ -302,9 +370,9 @@ sub check_config {
     while (@_) {
 	my ($key, $value) = (shift, shift);
 	if (defined $default_config->{$key}) {
-	    if ($key eq 'WITH') {
-		croak M10_usage_WITH() if (ref $value and
-					   ref $value ne 'ARRAY');
+	    if ($key =~ /^(WITH|USING)$/) {
+		croak M10_usage_WITH_USING()
+                  if (ref $value and ref $value ne 'ARRAY');
 		$value = [$value] unless ref $value;
 		$o->{CONFIG}{$key} = $value;
 		next;
@@ -315,7 +383,7 @@ sub check_config {
 		$value = abs_path($value);
 	    }
 	    elsif ($key eq 'NAME') {
-		croak M12_usage_NAME($value) 
+		croak M12_usage_NAME($value)
 		  unless $value =~ /^[a-zA-Z_](\w|::)*$/;
 	    }
 	    elsif ($key eq 'VERSION') {
@@ -335,6 +403,28 @@ sub check_config {
 #==============================================================================
 sub fold_options {
     my $o = shift;
+
+# The following small section of code seems, to me, to be unnecessary - which is the
+# reason that I've commented it out. I've left it here (including its associated comments)
+# in case it later becomes evident that there *is* good reason to include it. --sisyphus
+#
+## This bit tries to enable UNTAINT automatically if required when running the test suite.
+#    my $env_ha = $ENV{HARNESS_ACTIVE} || 0 ;
+#    my ($harness_active) = $env_ha =~ /(.*)/ ;
+#    if (($harness_active)&&(! $o->{CONFIG}{UNTAINT})){
+#            eval {
+#                    require Scalar::Util;
+#                    $o->{CONFIG}{UNTAINT} =
+#                      (Scalar::Util::tainted(Cwd::cwd()) ? 1 : 0) ;
+## Disable SAFEMODE in the test suite, we know what we are doing...
+#                    $o->{CONFIG}{SAFEMODE} = 0 ;
+#                    warn "\n-[tT] enabled for test suite.
+#Automatically setting UNTAINT=1 and SAFEMODE=0.\n"
+#                     unless $Inline::_TAINT_WARNING_ ;
+#                    $Inline::_TAINT_WARNING_ = 1 ;
+#            } ;
+#   }
+##
     $untaint = $o->{CONFIG}{UNTAINT} || 0;
     $safemode = (($o->{CONFIG}{SAFEMODE} == -1) ?
 		 ($untaint ? 1 : 0) :
@@ -349,7 +439,7 @@ sub fold_options {
     if ($o->{CONFIG}{AUTONAME} == -1) {
 	$o->{CONFIG}{AUTONAME} = length($o->{CONFIG}{NAME}) ? 0 : 1;
     }
-    $o->{API}{cleanup} = 
+    $o->{API}{cleanup} =
       ($o->{CONFIG}{CLEAN_AFTER_BUILD} and not $o->{CONFIG}{REPORTBUG});
 }
 
@@ -370,28 +460,40 @@ sub check_installed {
     return unless $o->{CONFIG}{VERSION};
     croak M26_error_version_without_name()
       unless $o->{CONFIG}{NAME};
-    my $realname = $o->{API}{pkg};
-    $realname =~ s|::|/|g;
-    $realname .= '.pm';
-    my $realpath = ($INC{$realname})
-      or croak M27_module_not_indexed($realname);
-    $realpath =~ s|^(.*)/\Q$realname\E$|$1|
+
+    my @pkgparts = split(/::/, $o->{API}{pkg});
+    my $realname = File::Spec->catfile(@pkgparts) . '.pm';
+    my $realname_unix = File::Spec::Unix->catfile(@pkgparts) . '.pm';
+    my $realpath = $INC{$realname_unix}
+      or croak M27_module_not_indexed($realname_unix);
+
+    my ($volume,$dir,$file) = File::Spec->splitpath($realpath);
+    my @dirparts = File::Spec->splitdir($dir);
+    pop @dirparts unless $dirparts[-1];
+    push @dirparts, $file;
+    my @endparts = splice(@dirparts, 0 - @pkgparts);
+
+    $dirparts[-1] = 'arch'
+      if $dirparts[-2] eq 'blib' && $dirparts[-1] eq 'lib';
+    File::Spec->catfile(@endparts) eq $realname
       or croak M28_error_grokking_path($realpath);
+    $realpath =
+      File::Spec->catpath($volume,File::Spec->catdir(@dirparts),"");
 
     $o->{API}{version} = $o->{CONFIG}{VERSION};
     $o->{API}{module} = $o->{CONFIG}{NAME};
     my @modparts = split(/::/,$o->{API}{module});
     $o->{API}{modfname} = $modparts[-1];
-    $o->{API}{modpname} = join('/',@modparts);
-    $realpath =~ s|\bblib[/\\]lib$|blib/arch|;
+    $o->{API}{modpname} = File::Spec->catdir(@modparts);
 
     my $suffix = $Config{dlext};
-    my $obj = "$realpath/auto/$o->{API}{modpname}/$o->{API}{modfname}.$suffix";
-    croak M30_error_no_obj($o->{CONFIG}{NAME}, $o->{API}{pkg}, 
+    my $obj = File::Spec->catfile($realpath,'auto',$o->{API}{modpname},
+                                  "$o->{API}{modfname}.$suffix");
+    croak M30_error_no_obj($o->{CONFIG}{NAME}, $o->{API}{pkg},
 			   $realpath) unless -f $obj;
 
-    @{$o->{CONFIG}}{qw( PRINT_INFO 
-			REPORTBUG 
+    @{$o->{CONFIG}}{qw( PRINT_INFO
+			REPORTBUG
 			FORCE_BUILD
 			_INSTALL_
 		      )} = (0, 0, 0, 0);
@@ -411,7 +513,7 @@ sub load {
 
     if ($o->{CONFIG}{_INSTALL_}) {
 	my $inline = "$o->{API}{modfname}.inl";
-	open INLINE, "> $inline" 
+	open INLINE, "> $inline"
 	  or croak M24_open_for_output_failed($inline);
 	print INLINE "*** AUTOGENERATED by Inline.pm ***\n\n";
 	print INLINE "This file satisfies the make dependency for ";
@@ -437,7 +539,7 @@ sub load {
 
 	package $module;
 	push \@$ {module}::ISA, qw(Exporter DynaLoader);
-        sub dl_load_flags { $global } 
+        sub dl_load_flags { $global }
 	${module}::->bootstrap;
 END
     croak M43_error_bootstrap($module, $@) if $@;
@@ -505,7 +607,7 @@ sub handle_shortcuts {
 	else {
 	    croak M48_usage_shortcuts($option);
 	}
-    }    
+    }
 }
 
 #==============================================================================
@@ -540,14 +642,17 @@ __END__
 sub receive_code {
     my $o = shift;
     my $code = shift;
-    
+
     croak M02_usage() unless (defined $code and $code);
 
     if (ref $code eq 'CODE') {
 	$o->{API}{code} = &$code;
     }
-    elsif ($code =~ m|[/\\]| and
-	   $code =~ m|^[\w/.-]+$|) {
+    elsif (ref $code eq 'ARRAY') {
+        $o->{API}{code} = join '', @$code;
+    }
+    elsif ($code =~ m|[/\\:]| and
+           $code =~ m|^[/\\:\w.\-\ \$\[\]<>]+$|) {
 	if (-f $code) {
 	    local ($/, *CODE);
 	    open CODE, "< $code" or croak M06_code_file_failed_open($code);
@@ -556,9 +661,6 @@ sub receive_code {
 	else {
 	    croak M07_code_file_does_not_exist($code);
 	}
-    } 
-    elsif (ref $code eq 'ARRAY') {
-	$o->{API}{code} = join '', @$code;
     }
     else {
 	$o->{API}{code} = $code;
@@ -573,8 +675,8 @@ sub read_inline_file {
     my ($lang, $pkg) = @{$o->{API}}{qw(language_id pkg)};
     my $langfile = uc($lang);
     croak M59_bad_inline_file($lang) unless $langfile =~ /^[A-Z]\w*$/;
-    croak M60_no_inline_files() 
-      unless (defined $INC{"Inline/Files.pm"} and
+    croak M60_no_inline_files()
+      unless (defined $INC{File::Spec::Unix->catfile("Inline","Files.pm")} and
 	      $Inline::Files::VERSION =~ /^\d\.\d\d$/ and
 	      $Inline::Files::VERSION ge '0.51');
     croak M61_not_parsed() unless $lang = Inline::Files::get_filename($pkg);
@@ -596,8 +698,8 @@ sub check_config_file {
     my ($DIRECTORY, %config);
     my $o = shift;
 
-    croak M14_usage_Config() if defined %main::Inline::Config::;
-    croak M63_no_source($o->{API}{pkg}) 
+    croak M14_usage_Config() if %main::Inline::Config::;
+    croak M63_no_source($o->{API}{pkg})
       if $o->{INLINE}{md5} eq $o->{API}{code};
 
     # First make sure we have the DIRECTORY
@@ -605,7 +707,8 @@ sub check_config_file {
 	croak M15_usage_install_directory()
 	  if $o->{CONFIG}{DIRECTORY};
 	my $cwd = Cwd::cwd();
-	$DIRECTORY = $o->{INLINE}{DIRECTORY} = "$cwd/_Inline";
+        $DIRECTORY =
+          $o->{INLINE}{DIRECTORY} = File::Spec->catdir($cwd, $did);
 	if (not -d $DIRECTORY) {
 	    _mkdir($DIRECTORY, 0777)
 	      or croak M16_DIRECTORY_mkdir_failed($DIRECTORY);
@@ -616,23 +719,34 @@ sub check_config_file {
 	  $o->{CONFIG}{DIRECTORY} || $o->find_temp_dir;
     }
 
-    $o->create_config_file($DIRECTORY) if not -e "$DIRECTORY/config";
+    $o->create_config_file($DIRECTORY)
+      if not -e File::Spec->catfile($DIRECTORY, $configuration_file);
 
-    open CONFIG, "< $DIRECTORY/config"
+    open CONFIG, "< ".File::Spec->catfile($DIRECTORY, $configuration_file)
       or croak M17_config_open_failed($DIRECTORY);
     my $config = join '', <CONFIG>;
     close CONFIG;
 
-    croak M62_invalid_config_file("$DIRECTORY/config")
+    croak M62_invalid_config_file(File::Spec->catfile($DIRECTORY, $configuration_file))
       unless $config =~ /^version :/;
-    ($config) = $config =~ /(.*)/s if UNTAINT;
+    if(UNTAINT) {
+      warn "In Inline::check_config_file(): Blindly untainting Inline configuration file information.\n" unless $o->{CONFIG}{NO_UNTAINT_WARN};
+      ($config) = $config =~ /(.*)/s;
+    }
 
     %config = Inline::denter->new()->undent($config);
     $Inline::languages = $config{languages};
 
+    {
+    no warnings ('numeric'); # These warnings were a pain with devel releases.
+                             # If there's a problem with the version number, the
+                             # error message will output $config{version} anyway.
     croak M18_error_old_version($config{version}, $DIRECTORY)
 	unless (defined $config{version} and
+                $config{version} =~ /TRIAL/ or
 		$config{version} >= 0.40);
+    } # numeric warnings re-enabled.
+
     croak M19_usage_language($o->{API}{language_id}, $DIRECTORY)
       unless defined $config{languages}->{$o->{API}{language_id}};
     $o->{API}{language} = $config{languages}->{$o->{API}{language_id}};
@@ -660,18 +774,24 @@ sub create_config_file {
     if (defined $o) {
 	($dir) = $dir =~ /(.*)/s if UNTAINT;
 	my $perl = $Config{perlpath};
+        $perl = $^X unless -f $perl;
 	($perl) = $perl =~ /(.*)/s if UNTAINT;
 	local $ENV{PERL5LIB} if defined $ENV{PERL5LIB};
 	local $ENV{PERL5OPT} if defined $ENV{PERL5OPT};
 	my $inline = $INC{'Inline.pm'};
-	$inline =~ s|\\|/|g;
-	$inline =~ s|/+|/|g;
-	$inline =~ s|/?Inline\.pm||;
-	$inline ||= '.';
-	my $INC = "-I$inline -I" . join(" -I", grep {(-d "$_/Inline" or 
-						      -d "$_/auto/Inline"
-						     )} @INC);
-	system "$perl $INC -MInline=_CONFIG_ -e1 $dir"
+        $inline ||= File::Spec->curdir();
+        my($v,$d,$f) = File::Spec->splitpath($inline);
+        $f = "" if $f eq 'Inline.pm';
+        $inline = File::Spec->catpath($v,$d,$f);
+        #my $INC = "-I$inline -I" .
+        #          join(" -I", grep {(-d File::Spec->catdir($_,"Inline") or
+        #                             -d File::Spec->catdir($_,"auto","Inline")
+	  #		            )} @INC);
+	#system "$perl $INC -MInline=_CONFIG_ -e1 $dir"
+        my @INC = map { "-I$_" }
+       ($inline,
+        grep {(-d File::Spec->catdir($_,"Inline") or -d File::Spec->catdir($_,"auto","Inline"))} @INC);
+       system $perl, @INC, "-MInline=_CONFIG_", "-e1", "$dir"
 	  and croak M20_config_creation_failed($dir);
 	return;
     }
@@ -680,9 +800,9 @@ sub create_config_file {
 	%languages, %types, %modules, %suffixes);
   LIB:
     for my $lib (@INC) {
-	next unless -d "$lib/Inline";
-	opendir LIB, "$lib/Inline" 
-	  or warn(M21_opendir_failed("$lib/Inline")), next;
+        next unless -d File::Spec->catdir($lib,"Inline");
+        opendir LIB, File::Spec->catdir($lib,"Inline")
+          or warn(M21_opendir_failed(File::Spec->catdir($lib,"Inline"))), next;
 	while ($mod = readdir(LIB)) {
 	    next unless $mod =~ /\.pm$/;
 	    $mod =~ s/\.pm$//;
@@ -692,9 +812,11 @@ sub create_config_file {
 		next;
 	    }
 	    next if $mod =~ /^(MakeMaker|denter|messages)$/;
-	    eval "require Inline::$mod;\$register=&Inline::${mod}::register";
+	    eval "require Inline::$mod;";
+            warn($@), next if $@;
+	    eval "\$register=&Inline::${mod}::register";
 	    next if $@;
-	    my $language = ($register->{language}) 
+	    my $language = ($register->{language})
 	      or warn(M22_usage_register($mod)), next;
 	    for (@{$register->{aliases}}) {
 		warn(M23_usage_alias_used($mod, $_, $languages{$_})), next
@@ -709,7 +831,7 @@ sub create_config_file {
 	closedir LIB;
     }
 
-    my $file = "$ARGV[0]/config";
+    my $file = File::Spec->catfile($ARGV[0], $configuration_file);
     open CONFIG, "> $file" or croak M24_open_for_output_failed($file);
     print CONFIG Inline::denter->new()
       ->indent(*version => $Inline::VERSION,
@@ -735,7 +857,8 @@ sub check_module {
     }
     elsif ($o->{API}{pkg} eq 'main') {
 	$module = $o->{API}{script};
-	$module =~ s|^.*[/\\](.*)$|$1|;
+        my($v,$d,$file) = File::Spec->splitpath($module);
+        $module = $file;
 	$module =~ s|\W|_|g;
 	$module =~ s|^_+||;
 	$module =~ s|_+$||;
@@ -752,8 +875,8 @@ sub check_module {
     my $auto_level = 2;
     while ($auto_level <= 5) {
 	if ($o->{CONFIG}{AUTONAME}) {
-	    $module2 = 
-	      $module . '_' . substr($o->{INLINE}{md5}, 0, 2**$auto_level);
+	    $module2 =
+	      $module . '_' . substr($o->{INLINE}{md5}, 0, 2 + $auto_level);
 	    $auto_level++;
 	} else {
 	    $module2 = $module;
@@ -762,15 +885,18 @@ sub check_module {
 	$o->{API}{module} = $module2;
 	my @modparts = split /::/, $module2;
 	$o->{API}{modfname} = $modparts[-1];
-	$o->{API}{modpname} = join('/',@modparts);
-	$o->{API}{build_dir} = 
-	  $o->{INLINE}{DIRECTORY} . '/build/' . $o->{API}{modpname};
-	$o->{API}{install_lib} = $o->{INLINE}{DIRECTORY} . '/lib';
+        $o->{API}{modpname} = File::Spec->catdir(@modparts);
+	$o->{API}{build_dir} =
+          File::Spec->catdir($o->{INLINE}{DIRECTORY},
+                             'build',$o->{API}{modpname});
+        $o->{API}{install_lib} =
+          File::Spec->catdir($o->{INLINE}{DIRECTORY}, 'lib');
 
-	my $inl = "$o->{API}{install_lib}/auto/" .
-	  "$o->{API}{modpname}/$o->{API}{modfname}.inl";
-	$o->{API}{location} = "$o->{API}{install_lib}/auto/" .
-	  "$o->{API}{modpname}/$o->{API}{modfname}.$o->{INLINE}{ILSM_suffix}";
+        my $inl = File::Spec->catfile($o->{API}{install_lib},"auto",
+                          $o->{API}{modpname},"$o->{API}{modfname}.inl");
+        $o->{API}{location} =
+          File::Spec->catfile($o->{API}{install_lib},"auto",$o->{API}{modpname},
+                              "$o->{API}{modfname}.$o->{INLINE}{ILSM_suffix}");
 	last unless -f $inl;
 	my %inl;
 	{   local ($/, *INL);
@@ -779,6 +905,8 @@ sub check_module {
 	}
 	next unless ($o->{INLINE}{md5} eq $inl{md5});
 	next unless ($inl{inline_version} ge '0.40');
+      next unless ($inl{Config}{version} eq $Config::Config{version});
+      next unless ($inl{Config}{archname} eq $Config::Config{archname});
 	unless (-f $o->{API}{location}) {
 	    warn <<END if $^W;
 Missing object file: $o->{API}{location}
@@ -801,7 +929,7 @@ sub install {
     my $o = shift;
 
     croak M64_install_not_c($o->{API}{language_id})
-      unless uc($o->{API}{language_id}) eq 'C';
+      unless uc($o->{API}{language_id}) =~ /^(C|CPP|Java|Python|Ruby|Lisp|Pdlpp)$/ ;
     croak M36_usage_install_main()
       if ($o->{API}{pkg} eq 'main');
     croak M37_usage_install_auto()
@@ -815,9 +943,9 @@ sub install {
 #	      $o->{CONFIG}{NAME} =~ /^$o->{API}{pkg}::\w(\w|::)+$/
 #	     );
 
-    my ($mod_name, $mod_ver, $ext_name, $ext_ver) = 
+    my ($mod_name, $mod_ver, $ext_name, $ext_ver) =
       ($o->{API}{pkg}, $ARGV[0], @{$o->{CONFIG}}{qw(NAME VERSION)});
-    croak M41_usage_install_version_mismatch($mod_name, $mod_ver, 
+    croak M41_usage_install_version_mismatch($mod_name, $mod_ver,
 					     $ext_name, $ext_ver)
       unless ($mod_ver eq $ext_ver);
     $o->{INLINE}{INST_ARCHLIB} = $ARGV[1];
@@ -826,16 +954,17 @@ sub install {
     $o->{API}{module} = $o->{CONFIG}{NAME};
     my @modparts = split(/::/,$o->{API}{module});
     $o->{API}{modfname} = $modparts[-1];
-    $o->{API}{modpname} = join('/',@modparts);
+    $o->{API}{modpname} = File::Spec->catdir(@modparts);
     $o->{API}{suffix} = $o->{INLINE}{ILSM_suffix};
-    $o->{API}{build_dir} = ( $o->{INLINE}{DIRECTORY} . '/build/' . 
-			     $o->{API}{modpname}
-			   );
+    $o->{API}{build_dir} = File::Spec->catdir($o->{INLINE}{DIRECTORY},'build',
+                                              $o->{API}{modpname});
     $o->{API}{directory} = $o->{INLINE}{DIRECTORY};
     my $cwd = Cwd::cwd();
-    $o->{API}{install_lib} = "$cwd/$o->{INLINE}{INST_ARCHLIB}";
-    $o->{API}{location} = "$o->{API}{install_lib}/auto/" .
-      "$o->{API}{modpname}/$o->{API}{modfname}.$o->{INLINE}{ILSM_suffix}";
+    $o->{API}{install_lib} =
+      File::Spec->catdir($cwd,$o->{INLINE}{INST_ARCHLIB});
+    $o->{API}{location} =
+      File::Spec->catfile($o->{API}{install_lib},"auto",$o->{API}{modpname},
+                          "$o->{API}{modfname}.$o->{INLINE}{ILSM_suffix}");
     unshift @::INC, $o->{API}{install_lib};
     $o->{INLINE}{object_ready} = 0;
 }
@@ -845,9 +974,11 @@ sub install {
 #==============================================================================
 sub write_inl_file {
     my $o = shift;
-    my $inl = "$o->{API}{install_lib}/auto/$o->{API}{modpname}/$o->{API}{modfname}.inl";
+    my $inl =
+      File::Spec->catfile($o->{API}{install_lib},"auto",$o->{API}{modpname},
+                          "$o->{API}{modfname}.inl");
     open INL, "> $inl"
-      or die "Can't create Inline validation file $inl";
+      or croak "Can't create Inline validation file $inl: $!";
     my $apiversion = $Config{apiversion} || $Config{xs_apiversion};
     print INL Inline::denter->new()
       ->indent(*md5, $o->{INLINE}{md5},
@@ -864,7 +995,7 @@ sub write_inl_file {
 	       *Config, { (map {($_,$Config{$_})}
 			   (qw( archname osname osvers
 				cc ccflags ld so version
-			      ))), 
+			      ))),
 			  (apiversion => $apiversion),
 			},
 	      );
@@ -889,18 +1020,26 @@ sub with_configs {
 }
 
 #==============================================================================
-# Blindly untaint tainted fields in Inline object.
+# Blindly untaint tainted fields in %ENV.
 #==============================================================================
 sub env_untaint {
     my $o = shift;
+    warn "In Inline::env_untaint() : Blindly untainting tainted fields in %ENV.\n" unless $o->{CONFIG}{NO_UNTAINT_WARN};
 
-    for (keys %ENV) {
-	($ENV{$_}) = $ENV{$_} =~ /(.*)/;
+    {
+    no warnings ('uninitialized'); # In case $ENV{$_} is set to undef.
+      for (keys %ENV) {
+	  ($ENV{$_}) = $ENV{$_} =~ /(.*)/;
+      }
     }
-    my $delim = $^O eq 'MSWin32' ? ';' : ':';
-    $ENV{PATH} = join $delim, grep {not /^\./ and
+
+    $ENV{PATH} = $^O eq 'MSWin32' ?
+                 join ';', grep {not /^\./ and -d $_
+				  } split /;/, $ENV{PATH}
+                 :
+                 join ':', grep {not /^\./ and -d $_ and
 				      not ((stat($_))[2] & 0022)
-				  } split $delim, $ENV{PATH};
+				  } split /:/, $ENV{PATH};
     map {($_) = /(.*)/} @INC;
 }
 #==============================================================================
@@ -908,7 +1047,7 @@ sub env_untaint {
 #==============================================================================
 sub obj_untaint {
     my $o = shift;
-
+    warn "In Inline::obj_untaint() : Blindly untainting tainted fields in Inline object.\n" unless $o->{CONFIG}{NO_UNTAINT_WARN};
     ($o->{INLINE}{ILSM_module}) = $o->{INLINE}{ILSM_module} =~ /(.*)/;
     ($o->{API}{build_dir}) = $o->{API}{build_dir} =~ /(.*)/;
     ($o->{CONFIG}{DIRECTORY}) = $o->{CONFIG}{DIRECTORY} =~ /(.*)/;
@@ -930,11 +1069,12 @@ sub clean_build {
 
     $prefix = $o->{INLINE}{DIRECTORY};
     opendir(BUILD, $prefix)
-      or die "Can't open build directory: $prefix for cleanup $!\n";
+      or croak "Can't open build directory: $prefix for cleanup $!\n";
 
     while ($dir = readdir(BUILD)) {
-	if ((-d "$prefix$dir") and ($dir =~ /\w{36,}/)) {
-	    $o->rmpath($prefix, $dir); 
+        my $maybedir = File::Spec->catdir($prefix,$dir);
+        if (($maybedir and -d $maybedir) and ($dir =~ /\w{36,}/)) {
+            $o->rmpath($prefix,$dir);
 	}
     }
 
@@ -996,20 +1136,21 @@ END
     {
 	no strict 'refs';
 	%versions = map {eval "use $_();"; ($_, $ {$_ . '::VERSION'})}
-	qw (Digest::MD5 Parse::RecDescent 
-	    ExtUtils::MakeMaker File::Path FindBin 
+	qw (Digest::MD5 Parse::RecDescent
+	    ExtUtils::MakeMaker File::Path FindBin
 	    Inline
 	   );
     }
 
     $o->mkpath($o->{API}{build_dir});
-    open REPORTBUG, "> $o->{API}{build_dir}/REPORTBUG"
-      or croak M24_open_for_output_failed("$o->{API}{build_dir}/REPORTBUG");
+    open REPORTBUG, "> ".File::Spec->catfile($o->{API}{build_dir},"REPORTBUG")
+      or croak M24_open_for_output_failed
+               (File::Spec->catfile($o->{API}{build_dir},"REPORTBUG"));
     %Inline::REPORTBUG_Inline_Object = ();
     %Inline::REPORTBUG_Perl_Config = ();
     %Inline::REPORTBUG_Module_Versions = ();
     print REPORTBUG Inline::denter->new()
-      ->indent(*REPORTBUG_Inline_Object, $o, 
+      ->indent(*REPORTBUG_Inline_Object, $o,
 	       *REPORTBUG_Perl_Config, \%Config::Config,
 	       *REPORTBUG_Module_Versions, \%versions,
 	      );
@@ -1029,7 +1170,7 @@ sub print_info {
 Information about the processing of your Inline $o->{API}{language_id} code:
 
 END
-    
+
     print STDERR <<END if ($o->{INLINE}{object_ready});
 Your module is already compiled. It is located at:
 $o->{API}{location}
@@ -1053,7 +1194,7 @@ and I\'ll install the executable as:
 $o->{API}{location}
 
 END
-    
+
     eval {
 	print STDERR $o->info;
     };
@@ -1083,13 +1224,13 @@ sub maker_utils {
 sub mkpath {
     use strict;
     my ($o, $mkpath) = @_;
-    my @parts = grep {$_} split(/\//,$mkpath);
-    my $path = ($parts[0] =~ /^[A-Za-z]:$/)
-      ? shift(@parts) . '/'  #MSWin32 Drive Letter (ie C:)
-	: '/';
+    my($volume,$dirs,$nofile) = File::Spec->splitpath($mkpath,1);
+    my @parts = File::Spec->splitdir($dirs);
+    my @done;
     foreach (@parts){
-	-d "$path$_" || _mkdir("$path$_", 0777);
-	$path .= "$_/";
+        push(@done,$_);
+        my $path = File::Spec->catpath($volume,File::Spec->catdir(@done),"");
+        -d $path || _mkdir($path, 0777);
     }
     croak M53_mkdir_failed($mkpath)
       unless -d $mkpath;
@@ -1102,13 +1243,13 @@ sub rmpath {
     use strict;
     my ($o, $prefix, $rmpath) = @_;
 # Nuke the target directory
-    _rmtree("$prefix$rmpath");
+    _rmtree(File::Spec->catdir($prefix ? ($prefix,$rmpath) : ($rmpath)));
 # Remove any empty directories underneath the requested one
-    my @parts = grep {$_} split(/\//,$rmpath);
-    pop @parts;
+    my @parts = File::Spec->splitdir($rmpath);
     while (@parts){
-	$rmpath = join '/', @parts;
-	rmdir "$prefix$rmpath"
+        $rmpath = File::Spec->catdir($prefix ? ($prefix,@parts) : @parts);
+        ($rmpath) = $rmpath =~ /(.*)/ if UNTAINT;
+        rmdir $rmpath
 	  or last; # rmdir failed because dir was not empty
 	pop @parts;
     }
@@ -1116,26 +1257,33 @@ sub rmpath {
 
 sub _rmtree {
     my($roots) = @_;
-    my(@files);
     $roots = [$roots] unless ref $roots;
     my($root);
     foreach $root (@{$roots}) {
-        $root =~ s#/\z##;
         if ( -d $root ) {
+            my(@names,@paths);
             if (opendir MYDIR, $root) {
-                @files = readdir MYDIR;
+                @names = readdir MYDIR;
                 closedir MYDIR;
             }
             else {
                 croak M21_opendir_failed($root);
             }
 
-            @files = map("$root/$_", grep $_!~/^\.{1,2}\z/s,@files);
-            _rmtree(\@files);
+            my $dot    = File::Spec->curdir();
+            my $dotdot = File::Spec->updir();
+            foreach my $name (@names) {
+                next if $name eq $dot or $name eq $dotdot;
+                my $maybefile = File::Spec->catfile($root,$name);
+                push(@paths,$maybefile),next if $maybefile and -f $maybefile;
+                push(@paths,File::Spec->catdir($root,$name));
+            }
+
+            _rmtree(\@paths);
 	    ($root) = $root =~ /(.*)/ if UNTAINT;
             rmdir($root) or croak M54_rmdir_failed($root);
         }
-        else { 
+        else {
 	    ($root) = $root =~ /(.*)/ if UNTAINT;
 	    unlink($root) or croak M55_unlink_failed($root);
         }
@@ -1148,12 +1296,12 @@ sub _rmtree {
 my $TEMP_DIR;
 sub find_temp_dir {
     return $TEMP_DIR if $TEMP_DIR;
-    
+
     my ($temp_dir, $home, $bin, $cwd, $env);
     $temp_dir = '';
     $env = $ENV{PERL_INLINE_DIRECTORY} || '';
     $home = $ENV{HOME} ? abs_path($ENV{HOME}) : '';
-    
+
     if ($env and
 	-d $env and
 	-w $env) {
@@ -1161,43 +1309,45 @@ sub find_temp_dir {
     }
     elsif ($cwd = abs_path('.') and
 	   $cwd ne $home and
-	   -d "$cwd/.Inline/" and
-	   -w "$cwd/.Inline/") {
-	$temp_dir = "$cwd/.Inline";
-    }
-    elsif (require FindBin and
-           $bin = $FindBin::Bin and
-	   -d "$bin/.Inline/" and
-	   -w "$bin/.Inline/") {
-	$temp_dir = "$bin/.Inline";
-    } 
-    elsif ($home and
-	   -d "$home/.Inline/" and
-	   -w "$home/.Inline/") {
-	$temp_dir = "$home/.Inline";
-    } 
-    elsif (defined $cwd and $cwd and
-	   -d "$cwd/_Inline/" and
-	   -w "$cwd/_Inline/") {
-	$temp_dir = "$cwd/_Inline";
-    }
-    elsif (defined $bin and $bin and
-	   -d "$bin/_Inline/" and
-	   -w "$bin/_Inline/") {
-	$temp_dir = "$bin/_Inline";
-    } 
-    elsif (defined $cwd and $cwd and
-	   -d $cwd and
-	   -w $cwd and
-	   _mkdir("$cwd/_Inline", 0777)) {
-	$temp_dir = "$cwd/_Inline";
-    }
-    elsif (defined $bin and $bin and
-	   -d $bin and
-	   -w $bin and
-	   _mkdir("$bin/_Inline", 0777)) {
-	$temp_dir = "$bin/_Inline";
+           -d File::Spec->catdir($cwd,".Inline") and
+           -w File::Spec->catdir($cwd,".Inline")) {
+        $temp_dir = File::Spec->catdir($cwd,".Inline");
     }
+       else {
+               require FindBin ;
+        if ($bin = $FindBin::Bin and
+               -d File::Spec->catdir($bin,".Inline") and
+               -w File::Spec->catdir($bin,".Inline")) {
+            $temp_dir = File::Spec->catdir($bin,".Inline");
+        }
+        elsif ($home and
+               -d File::Spec->catdir($home,".Inline") and
+               -w File::Spec->catdir($home,".Inline")) {
+            $temp_dir = File::Spec->catdir($home,".Inline");
+        }
+        elsif (defined $cwd and $cwd and
+               -d File::Spec->catdir($cwd, $did) and
+               -w File::Spec->catdir($cwd, $did)) {
+            $temp_dir = File::Spec->catdir($cwd, $did);
+        }
+        elsif (defined $bin and $bin and
+               -d File::Spec->catdir($bin, $did) and
+               -w File::Spec->catdir($bin, $did)) {
+            $temp_dir = File::Spec->catdir($bin, $did);
+        }
+        elsif (defined $cwd and $cwd and
+          -d $cwd and
+          -w $cwd and
+               _mkdir(File::Spec->catdir($cwd, $did), 0777)) {
+            $temp_dir = File::Spec->catdir($cwd, $did);
+        }
+        elsif (defined $bin and $bin and
+          -d $bin and
+          -w $bin and
+               _mkdir(File::Spec->catdir($bin, $did), 0777)) {
+            $temp_dir = File::Spec->catdir($bin, $did);
+        }
+       }
 
     croak M56_no_DIRECTORY_found()
       unless $temp_dir;
@@ -1208,7 +1358,7 @@ sub _mkdir {
     my $dir = shift;
     my $mode = shift || 0777;
     ($dir) = ($dir =~ /(.*)/) if UNTAINT;
-    $dir =~ s|[/\\]$||;
+    $dir =~ s|[/\\:]$||;
     return mkdir($dir, $mode);
 }
 
@@ -1223,7 +1373,7 @@ sub _mkdir {
 sub M01_usage_use {
     my ($module) = @_;
     return <<END;
-It is invalid to use '$module' directly. Please consult the Inline 
+It is invalid to use '$module' directly. Please consult the Inline
 documentation for more information.
 
 END
@@ -1299,7 +1449,7 @@ END
 sub M07_code_file_does_not_exist {
     my ($file) = @_;
     return <<END;
-Inline assumes '$file' is a filename, 
+Inline assumes '$file' is a filename,
 and that file does not exist.
 
 END
@@ -1321,9 +1471,10 @@ Marker '$marker' does not match Inline '$lang' section.
 END
 }
 
-sub M10_usage_WITH {
+sub M10_usage_WITH_USING {
     return <<END;
-Config option WITH must be a module name or an array ref of module names.
+Config option WITH or USING must be a module name or an array ref
+of module names.
 
 END
 }
@@ -1348,7 +1499,7 @@ sub M13_usage_VERSION {
     my ($version) = @_;
     return <<END;
 Invalid value for VERSION config option: '$version'
-Must be of the form '#.##'. 
+Must be of the form '#.##'.
 (Should also be specified as a string rather than a floating point number)
 
 END
@@ -1383,8 +1534,9 @@ END
 
 sub M17_config_open_failed {
     my ($dir) = @_;
+    my $file = File::Spec->catfile(${dir}, $configuration_file);
     return <<END;
-Can't open ${dir}/config for input.
+Can't open ${file} for input.
 
 END
 #'
@@ -1394,7 +1546,7 @@ sub M18_error_old_version {
     my ($old_version, $directory) = @_;
     $old_version ||= '???';
     return <<END;
-You are using Inline version $Inline::VERSION with a directory that was 
+You are using Inline version $Inline::VERSION with a directory that was
 configured by Inline version $old_version. This version is no longer supported.
 Please delete the following directory and try again:
 
@@ -1405,16 +1557,17 @@ END
 
 sub M19_usage_language {
     my ($language, $directory) = @_;
+    require Config;
     return <<END;
 Error. You have specified '$language' as an Inline programming language.
 
 I currently only know about the following languages:
-    ${ defined $Inline::languages ? 
+    ${ defined $Inline::languages ?
        \ join(', ', sort keys %$Inline::languages) : \ ''
      }
 
 If you have installed a support module for this language, try deleting the
-config file from the following Inline DIRECTORY, and run again:
+config-${Config::Config{'archname'}}-$] file from the following Inline DIRECTORY, and run again:
 
     $directory
 
@@ -1423,8 +1576,9 @@ END
 
 sub M20_config_creation_failed {
     my ($dir) = @_;
+    my $file = File::Spec->catfile(${dir}, $configuration_file);
     return <<END;
-Failed to autogenerate ${dir}/config.
+Failed to autogenerate ${file}.
 
 END
 }
@@ -1536,7 +1690,7 @@ END
 sub M31_inline_open_failed {
     my ($file) = @_;
     return <<END;
-Can't open Inline validate file: 
+Can't open Inline validate file:
 
     $file
 
@@ -1701,8 +1855,8 @@ sub M48_usage_shortcuts {
 Invalid shortcut '$shortcut' specified.
 
 Valid shortcuts are:
-    VERSION, INFO, FORCE, NOCLEAN, CLEAN, UNTAINT, SAFE, UNSAFE, 
-    GLOBAL and REPORTBUG
+    VERSION, INFO, FORCE, NOCLEAN, CLEAN, UNTAINT, SAFE, UNSAFE,
+    GLOBAL, NOISY and REPORTBUG
 
 END
 }
@@ -1716,7 +1870,7 @@ Either use the DIRECTORY option or turn off SAFEMODE.
 
 END
       ($terminate ? <<END : "");
-Since you are running as the a privledged user, Inline.pm is terminating.
+Since you are running as a privileged user, Inline.pm is terminating.
 
 END
 }
@@ -1748,7 +1902,7 @@ END
 sub M54_rmdir_failed {
     my ($dir) = @_;
     return <<END;
-Can't remove directory '$dir': 
+Can't remove directory '$dir':
 
 $!
 
@@ -1759,7 +1913,7 @@ END
 sub M55_unlink_failed {
     my ($file) = @_;
     return <<END;
-Can't unlink file '$file': 
+Can't unlink file '$file':
 
 $!
 
@@ -1787,7 +1941,7 @@ END
 
 sub M58_site_install {
     return <<END;
-You have specified the SITE_INSTALL command. Support for this option has 
+You have specified the SITE_INSTALL command. Support for this option has
 been removed from Inline since version 0.40. It has been replaced by the
 use of Inline::MakeMaker in your Makefile.PL. Please see the Inline
 documentation for more help on creating and installing Inline based modules.
@@ -1798,7 +1952,7 @@ END
 sub M59_bad_inline_file {
     my ($lang) = @_;
     return <<END;
-Could not find any Inline source code for the '$lang' language using 
+Could not find any Inline source code for the '$lang' language using
 the Inline::Files module.
 
 END
@@ -1827,8 +1981,8 @@ You are using a config file that was created by an older version of Inline:
     $config
 
 This file and all the other components in its directory are no longer valid
-for this version of Inline. The best thing to do is simply delete all the 
-contents of the directory and let Inline rebuild everything for you. Inline 
+for this version of Inline. The best thing to do is simply delete all the
+contents of the directory and let Inline rebuild everything for you. Inline
 will do this automatically when you run your programs.
 
 END
@@ -1848,7 +2002,7 @@ sub M64_install_not_c {
     return <<END;
 Invalid attempt to install an Inline module using the '$lang' language.
 
-Only C based modules are currently supported.
+Only C and CPP (C++) based modules are currently supported.
 
 END
 }
